@@ -134,12 +134,14 @@ static int mncc_write(struct mncc_connection *conn, struct gsm_mncc *mncc)
 {
 	int rc;
 
+	LOGP(DMNCC, LOGL_DEBUG, "tx MNCC %s with SDP=%s\n", osmo_mncc_name(mncc->msg_type),
+	     osmo_quote_str(mncc->sdp, -1));
+
 	/*
 	 * TODO: we need to put cause in here for release or such? shall we return a
 	 * static struct?
 	 */
 	rc = write(conn->fd.fd, mncc, sizeof(*mncc));
-	LOGP(DMNCC, LOGL_DEBUG, "MNCC sent message type: %s\n", osmo_mncc_name(mncc->msg_type));
 	if (rc != sizeof(*mncc)) {
 		LOGP(DMNCC, LOGL_ERROR, "Failed to send message for call(%u)\n", mncc->callref);
 		close_connection(conn);
@@ -160,8 +162,10 @@ static int mncc_rtp_write(struct mncc_connection *conn, struct gsm_mncc_rtp *rtp
 {
 	int rc;
 
+	LOGP(DMNCC, LOGL_DEBUG, "tx MNCC %s with SDP=%s\n", osmo_mncc_name(rtp->msg_type),
+	     osmo_quote_str(rtp->sdp, -1));
+
 	rc = write(conn->fd.fd, rtp, sizeof(*rtp));
-	LOGP(DMNCC, LOGL_DEBUG, "MNCC sent message type: %s\n", osmo_mncc_name(rtp->msg_type));
 	if (rc != sizeof(*rtp)) {
 		LOGP(DMNCC, LOGL_ERROR, "Failed to send message for call(%u): %d\n", rtp->callref, rc);
 		close_connection(conn);
@@ -170,12 +174,14 @@ static int mncc_rtp_write(struct mncc_connection *conn, struct gsm_mncc_rtp *rtp
 	return rc;
 }
 
-static int mncc_rtp_send(struct mncc_connection *conn, uint32_t msg_type, uint32_t callref)
+static int mncc_rtp_send(struct mncc_connection *conn, uint32_t msg_type, uint32_t callref, const char *sdp)
 {
 	struct gsm_mncc_rtp mncc = { 0, };
 
 	mncc.msg_type = msg_type;
 	mncc.callref = callref;
+	if (sdp)
+		OSMO_STRLCPY_ARRAY(mncc.sdp, sdp);
 
 	return mncc_rtp_write(conn, &mncc);
 }
@@ -195,6 +201,10 @@ static bool send_rtp_connect(struct mncc_call_leg *leg, struct call_leg *other)
 	mncc.callref = leg->callref;
 	mncc.addr = other->addr;
 	mncc.payload_type = other->payload_type;
+
+	/* Forward whichever SDP was last received on the other call leg */
+	OSMO_STRLCPY_ARRAY(mncc.sdp, other->rx_sdp);
+
 	/*
 	 * FIXME: mncc.payload_msg_type should already be compatible.. but
 	 * payload_type should be different..
@@ -408,6 +418,8 @@ static void check_rtp_connect(struct mncc_connection *conn, const char *buf, int
 		return;
 	}
 
+	call_leg_rx_sdp(&leg->base, rtp->sdp);
+
 	/* extract information about where the RTP is */
 	if (rtp->addr.ss_family != AF_UNSPEC ||
 	    osmo_sockaddr_port((const struct sockaddr *)&rtp->addr) != 0 ||
@@ -446,6 +458,7 @@ static void check_rtp_create(struct mncc_connection *conn, const char *buf, int
 	leg->base.addr = rtp->addr;
 	leg->base.payload_type = rtp->payload_type;
 	leg->base.payload_msg_type = rtp->payload_msg_type;
+	call_leg_rx_sdp(&leg->base, rtp->sdp);
 
 	/* TODO.. now we can continue with the call */
 	LOGP(DMNCC, LOGL_INFO,
@@ -485,6 +498,8 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 	const struct gsm_mncc_number *called;
 	struct call *call;
 	struct mncc_call_leg *leg;
+	struct call_leg *other_leg;
+	const char *sdp = NULL;
 	struct osmo_gcr_parsed gcr;
 
 	if (rc < sizeof(*data)) {
@@ -557,6 +572,7 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 	memcpy(&leg->called, called, sizeof(leg->called));
 	memcpy(&leg->calling, &data->calling, sizeof(leg->calling));
 	memcpy(&leg->imsi, data->imsi, sizeof(leg->imsi));
+	call_leg_rx_sdp(&leg->base, data->sdp);
 
 	if (data->fields & MNCC_F_GCR) {
 		leg->base.call->gcr_present = true;
@@ -567,8 +583,14 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 		"Created call(%u) with MNCC leg(%u) IMSI(%.16s)\n",
 		call->id, leg->callref, data->imsi);
 
+	other_leg = call_leg_other(&leg->base);
+	if (other_leg && *other_leg->rx_sdp && other_leg->rx_sdp_changed) {
+		sdp = other_leg->rx_sdp;
+		other_leg->rx_sdp_changed = false;
+	}
+
 	start_cmd_timer(leg, MNCC_RTP_CREATE);
-	mncc_rtp_send(conn, MNCC_RTP_CREATE, data->callref);
+	mncc_rtp_send(conn, MNCC_RTP_CREATE, data->callref, sdp);
 }
 
 /*! Find MNCC Call leg by given MNCC message
@@ -672,6 +694,8 @@ static void check_stp_cmpl_ind(struct mncc_connection *conn, const char *buf, in
 	if (!leg)
 		return;
 
+	call_leg_rx_sdp(&leg->base, data->sdp);
+
 	LOGP(DMNCC, LOGL_INFO, "leg(%u) is now connected.\n", leg->callref);
 	stop_cmd_timer(leg, MNCC_SETUP_COMPL_IND);
 	leg->state = MNCC_CC_CONNECTED;
@@ -701,17 +725,28 @@ static void check_cnf_ind(struct mncc_connection *conn, const char *buf, int rc)
 {
 	const struct gsm_mncc *data;
 	struct mncc_call_leg *leg;
+	struct call_leg *other_leg;
+	const char *sdp = NULL;
 
 	leg = find_leg(conn, buf, rc, &data);
 	if (!leg)
 		return;
 
+	call_leg_rx_sdp(&leg->base, data->sdp);
+
+	/* Forward SDP received from the other side */
+	other_leg = call_leg_other(&leg->base);
+	if (other_leg && *other_leg->rx_sdp && other_leg->rx_sdp_changed) {
+		sdp = other_leg->rx_sdp;
+		other_leg->rx_sdp_changed = false;
+	}
+
 	LOGP(DMNCC, LOGL_DEBUG,
 		"leg(%u) confirmed. creating RTP socket.\n",
 		leg->callref);
 
 	start_cmd_timer(leg, MNCC_RTP_CREATE);
-	mncc_rtp_send(conn, MNCC_RTP_CREATE, data->callref);
+	mncc_rtp_send(conn, MNCC_RTP_CREATE, data->callref, sdp);
 }
 
 static void check_alrt_ind(struct mncc_connection *conn, const char *buf, int rc)
@@ -724,6 +759,8 @@ static void check_alrt_ind(struct mncc_connection *conn, const char *buf, int rc
 	if (!leg)
 		return;
 
+	call_leg_rx_sdp(&leg->base, data->sdp);
+
 	LOGP(DMNCC, LOGL_DEBUG,
 		"leg(%u) is alerting.\n", leg->callref);
 
@@ -802,6 +839,8 @@ static void check_stp_cnf(struct mncc_connection *conn, const char *buf, int rc)
 	if (!leg)
 		return;
 
+	call_leg_rx_sdp(&leg->base, data->sdp);
+
 	LOGP(DMNCC, LOGL_DEBUG, "leg(%u) setup completed\n", leg->callref);
 
 	other_leg = call_leg_other(&leg->base);
@@ -831,6 +870,8 @@ static void check_dtmf_start(struct mncc_connection *conn, const char *buf, int
 	if (!leg)
 		return;
 
+	call_leg_rx_sdp(&leg->base, data->sdp);
+
 	LOGP(DMNCC, LOGL_DEBUG, "leg(%u) DTMF key=%c\n", leg->callref, data->keypad);
 
 	other_leg = call_leg_other(&leg->base);
@@ -853,6 +894,8 @@ static void check_dtmf_stop(struct mncc_connection *conn, const char *buf, int r
 	if (!leg)
 		return;
 
+	call_leg_rx_sdp(&leg->base, data->sdp);
+
 	LOGP(DMNCC, LOGL_DEBUG, "leg(%u) DTMF key=%c\n", leg->callref, data->keypad);
 
 	mncc_fill_header(&out_mncc, MNCC_STOP_DTMF_RSP, leg->callref);
@@ -945,11 +988,17 @@ int mncc_create_remote_leg(struct mncc_connection *conn, struct call *call)
 		msgb_free(msg);
 	}
 
+	/* The call->initial leg is a SIP call leg that starts an MT call. There was SDP received in the SIP INVITE that
+	 * started this call. This here will be the call->remote, always forwarding the SDP that came in on
+	 * call->initial. */
+	if (call->initial && call->initial->rx_sdp_changed) {
+		OSMO_STRLCPY_ARRAY(mncc.sdp, call->initial->rx_sdp);
+		call->initial->rx_sdp_changed = false;
+	}
+
 	/*
 	 * TODO/FIXME:
-	 *  - Determine/request channel based on offered audio codecs
 	 *  - Screening, redirect?
-	 *  - Synth. the bearer caps based on codecs?
 	 */
 	rc = mncc_write(conn, &mncc);
 	if (rc != sizeof(mncc)) {
@@ -983,6 +1032,53 @@ static void mncc_reconnect(void *data)
 	conn->state = MNCC_WAIT_VERSION;
 }
 
+static inline void log_mncc(const char *label, const uint8_t *buf, size_t buflen)
+{
+	uint32_t msg_type;
+	const struct gsm_mncc *gsm_mncc;
+	const struct gsm_mncc_rtp *gsm_mncc_rtp;
+	const char *sdp = NULL;
+
+	/* Any size errors will be logged elsewhere already, so just exit here if the buffer is too small. */
+	if (buflen < 4)
+		return;
+	memcpy(&msg_type, buf, 4);
+
+	switch (msg_type) {
+	case MNCC_SETUP_IND:
+	case MNCC_DISC_IND:
+	case MNCC_REL_IND:
+	case MNCC_REJ_IND:
+	case MNCC_REL_CNF:
+	case MNCC_SETUP_COMPL_IND:
+	case MNCC_SETUP_CNF:
+	case MNCC_CALL_CONF_IND:
+	case MNCC_ALERT_IND:
+	case MNCC_HOLD_IND:
+	case MNCC_RETRIEVE_IND:
+		if (buflen < sizeof(gsm_mncc))
+			return;
+		gsm_mncc = (void *)buf;
+		sdp = gsm_mncc->sdp;
+		break;
+	case MNCC_RTP_CREATE:
+	case MNCC_RTP_CONNECT:
+		if (buflen < sizeof(gsm_mncc_rtp))
+			return;
+		gsm_mncc_rtp = (void *)buf;
+		sdp = gsm_mncc_rtp->sdp;
+		break;
+
+	default:
+		break;
+	}
+	if (sdp)
+		LOGP(DMNCC, LOGL_DEBUG, "%sMNCC %s with SDP=%s\n", label, osmo_mncc_name(msg_type),
+		     osmo_quote_str(sdp, -1));
+	else
+		LOGP(DMNCC, LOGL_DEBUG, "%sMNCC %s\n", label, osmo_mncc_name(msg_type));
+}
+
 /* osmo-fd read call-back for MNCC socket: read MNCC message + dispatch it */
 static int mncc_data(struct osmo_fd *fd, unsigned int what)
 {
@@ -1002,10 +1098,10 @@ static int mncc_data(struct osmo_fd *fd, unsigned int what)
 		goto bad_data;
 	}
 
+	log_mncc("rx ", (void *)buf, rc);
+
+	/* Handle the received MNCC message */
 	memcpy(&msg_type, buf, 4);
-
-	LOGP(DMNCC, LOGL_DEBUG, "MNCC rcvd message type: %s\n", osmo_mncc_name(msg_type));
-
 	switch (msg_type) {
 	case MNCC_SOCKET_HELLO:
 		check_hello(conn, buf, rc);
