@@ -46,6 +46,12 @@ static void sip_dtmf_call(struct call_leg *_leg, int keypad);
 static void sip_hold_call(struct call_leg *_leg);
 static void sip_retrieve_call(struct call_leg *_leg);
 
+static const char *sip_get_sdp(const sip_t *sip)
+{
+	if (!sip || !sip->sip_payload)
+		return NULL;
+	return sip->sip_payload->pl_data;
+}
 
 /* Find a SIP Call leg by given nua_handle */
 static struct sip_call_leg *sip_find_leg(nua_handle_t *nh)
@@ -137,12 +143,7 @@ static void new_call(struct sip_agent *agent, nua_handle_t *nh,
 	}
 
 	call = call_sip_create();
-	if (!call) {
-		LOGP(DSIP, LOGL_ERROR, "No supported codec.\n");
-		nua_respond(nh, SIP_500_INTERNAL_SERVER_ERROR, TAG_END());
-		nua_handle_destroy(nh);
-		return;
-	}
+	OSMO_ASSERT(call);
 
 	/* Decode Decode the Global Call Reference (if present) */
 	if (xgcr_hdr_present) {
@@ -201,6 +202,8 @@ static void new_call(struct sip_agent *agent, nua_handle_t *nh,
 	nua_handle_bind(nh, leg);
 	leg->sdp_payload = talloc_strdup(leg, sip->sip_payload->pl_data);
 
+	call_leg_rx_sdp(&leg->base, sip_get_sdp(sip));
+
 	app_route_call(call,
 			talloc_strdup(leg, from),
 			talloc_strdup(leg, to));
@@ -241,6 +244,8 @@ static void sip_handle_reinvite(struct sip_call_leg *leg, nua_handle_t *nh, cons
 	     osmo_sockaddr_ntop((struct sockaddr*)&prev_addr, ip_addr),
 	     osmo_sockaddr_port((struct sockaddr*)&prev_addr));
 
+	call_leg_rx_sdp(&leg->base, sip_get_sdp(sip));
+
 	if (mode == sdp_sendonly) {
 		/* SIP side places call on HOLD */
 		sdp = sdp_create_file(leg, other, sdp_recvonly);
@@ -353,13 +358,15 @@ static int status2cause(int status)
 
 void nua_callback(nua_event_t event, int status, char const *phrase, nua_t *nua, nua_magic_t *magic, nua_handle_t *nh, nua_hmagic_t *hmagic, sip_t const *sip, tagi_t tags[])
 {
-	LOGP(DSIP, LOGL_DEBUG, "SIP event[%s] status(%d) phrase(%s) %p\n",
-		nua_event_name(event), status, phrase, hmagic);
+	LOGP(DSIP, LOGL_DEBUG, "SIP event[%s] status(%d) phrase(%s) SDP(%s) %p\n",
+		nua_event_name(event), status, phrase, sip_get_sdp(sip), hmagic);
 
 	if (event == nua_r_invite) {
 		struct sip_call_leg *leg;
 		leg = (struct sip_call_leg *) hmagic;
 
+		call_leg_rx_sdp(&leg->base, sip_get_sdp(sip));
+
 		/* MT call is moving forward */
 
 		/* The dialogue is now confirmed */
@@ -401,8 +408,10 @@ void nua_callback(nua_event_t event, int status, char const *phrase, nua_t *nua,
 		 * respond to the re-INVITE query. */
 		if (sip->sip_payload && sip->sip_payload->pl_data) {
 			struct sip_call_leg *leg = sip_find_leg(nh);
-			if (leg)
+			if (leg) {
+				call_leg_rx_sdp(&leg->base, sip_get_sdp(sip));
 				sip_handle_reinvite(leg, nh, sip);
+			}
 		}
 	} else if (event == nua_r_bye || event == nua_r_cancel) {
 		/* our bye or hang up is answered */
@@ -428,10 +437,12 @@ void nua_callback(nua_event_t event, int status, char const *phrase, nua_t *nua,
 
 		if (status == 100) {
 			struct sip_call_leg *leg = sip_find_leg(nh);
-			if (leg)
+			if (leg) {
+				call_leg_rx_sdp(&leg->base, sip_get_sdp(sip));
 				sip_handle_reinvite(leg, nh, sip);
-			else
+			} else {
 				new_call((struct sip_agent *) magic, nh, sip);
+			}
 		}
 	} else if (event == nua_i_cancel) {
 		struct sip_call_leg *leg;
@@ -526,6 +537,7 @@ static void sip_ring_call(struct call_leg *_leg)
 	OSMO_ASSERT(_leg->type == CALL_TYPE_SIP);
 	leg = (struct sip_call_leg *) _leg;
 
+	/* 180 Ringing should not contain any SDP. */
 	nua_respond(leg->nua_handle, SIP_180_RINGING, TAG_END());
 }
 
